from typing import Type
from celery import Celery

from pytest import mark
import pytest


from celery_redis_poll import install_redis_poll_backend, PollingRedisBackend, PollingRedisClusterBackend


@mark.parametrize("protocol, expected_backend", [
    ("redispoll", PollingRedisBackend),
    ("redisclusterpoll", PollingRedisClusterBackend),
    ("redisspoll", PollingRedisBackend),
])
def test_redis_cluster_backend_installation(
    protocol: str, expected_backend: Type,
):
    install_redis_poll_backend()

    # Initialize Celery app with RedisClusterBackend
    app = Celery(
        'test_app',
        broker='redis://localhost:6379/0',
        backend=f'{protocol}://localhost:6379/0'
    )

    # Check if the backend is set correctly
    assert isinstance(app.backend, expected_backend), f"Backend is not set to {expected_backend}"

if __name__ == "__main__":
    pytest.main()


__version__ = '0.1.2'

from celery_redis_poll.backend import PollingRedisBackend, PollingRedisClusterBackend

def install_redis_poll_backend():
    from celery.app.backends import BACKEND_ALIASES
    BACKEND_ALIASES['redispoll'] = 'celery_redis_poll.backend:PollingRedisBackend'
    BACKEND_ALIASES['redisclusterpoll'] = 'celery_redis_poll.backend:PollingRedisClusterBackend'
    BACKEND_ALIASES['redisspoll'] = 'celery_redis_poll.backend:PollingRedisBackend'

__all__ = ['PollingRedisBackend', 'PollingRedisClusterBackend', 'install_redis_poll_backend']



from celery.backends.redis import RedisBackend
from typing import Any, Optional
from celery_redis_cluster_backend import RedisClusterBackend


class PollingRedisBackend(RedisBackend):
    """
    Disables pub/sub for getting task results and instead uses polling.
    """
    def _set(self, key: str, value: str) -> None:
        """
        Simply set value in Redis, do not publish.
        :param key:
        :param value:
        :return:
        """
        if self.expires:
            self.client.setex(key, self.expires, value)
        else:
            self.client.set(key, value)

    def on_task_call(self, *args: Any, **kwargs: Any) -> None:
        pass

    def wait_for_pending(
        self, result_ids: list[str], timeout: Optional[int] = None, interval: float = 0.5, no_ack: bool = True
    ) -> list[dict]:
        """
        Overrides wait_for_pending to use polling instead of Pub/Sub.
        """
        # Poll Redis for results using pipeline
        start_time = self.app.now()
        while True:
            pipeline = self.client.pipeline()
            for task_id in result_ids:
                pipeline.get(self.get_key_for_task(task_id))
            results = pipeline.execute()

            if all(results):
                return [self.meta_from_decoded(self.decode_result(result)) for result in results]

            if timeout and (self.app.now() - start_time).total_seconds() >= timeout:
                raise TimeoutError("Timeout while waiting for task results.")

            self.app.sleep(interval)


class PollingRedisClusterBackend(PollingRedisBackend, RedisClusterBackend):
    """
    Same as PollingRedisBackend but with ReidsCluster for client.
    """
    pass


from setuptools import setup, find_packages

setup(
    name="celery-redis-poll",
    version="0.1.2",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    install_requires=[
        "celery>=5.3.0",
        "redis>=4.5.0",
        "celery-redis-cluster>=0.1.6",
    ],
    extras_require={
        'dev': [
            'pytest>=6.0.0',
        ],
    },
    author="Lev Neiman",
    author_email="lev.neiman@gmail.com",
    description="A Redis Backend for Celery with Polling",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/lan17/celery-redis-poll",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    python_requires=">=3.8",
)
